@@ -140,8 +140,8 @@ static int usage(int code)
 		"                       [SRC]:PORT [DST]:PORT\n", prognm);
 	fprintf(stderr, "\n"
 		"Options:\n"
-		" -b, --bind=IP            Force specific IP to bind() to when listening\n"
-		"                          for incoming connections\n"
+		" -b, --bind=IP            Force specific IP to bind() to when listening for\n"
+		"                          incoming connections.  Not applicable with -p\n"
 		" -h, --help               Show this help text\n"
 #ifndef NO_FTP
 		" -f, --ftp=TYPE           Redirect FTP connections.  Where type is\n"
@@ -155,7 +155,7 @@ static int usage(int code)
 #endif
 		" -l, --loglevel=LEVEL     Set log level: none, err, notice*, info, debug\n"
 		" -n, --foreground         Run in foreground, do not detach from terminal\n"
-		" -p, --transproxy         run in linux's transparent proxy mode\n"
+		" -p, --transproxy         Run in Linux's transparent proxy mode\n"
 		" -s, --syslog             Log messages to syslog\n"
 		" -t, --timeout=SEC        Set timeout to SEC seconds, default off (0)\n"
 		" -v, --version            Show program version\n"
@@ -196,29 +196,35 @@ static int loglvl(char *level)
 	return atoi(level);
 }
 
-static int parse_ipport(char *arg, char *buf, size_t len)
+static int parse_port(char *arg)
 {
 	int port;
-	char *ptr;
 	struct servent *s;
 
+	s = getservbyname(arg, "tcp");
+	if (s != NULL)
+		port = ntohs(s->s_port);
+	else
+		port = atoi(arg);
+
+	return port;
+}
+
+static int parse_ipport(char *arg, char *buf, size_t len)
+{
+	char *port;
+
 	if (!arg || !buf || !len)
 		return -1;
 
-	ptr = strchr(arg, ':');
-	if (!ptr)
+	port = strchr(arg, ':');
+	if (!port)
 		return -1;
+	*port++ = 0;
 
-	*ptr++ = 0;
 	snprintf(buf, len, "%s", arg);
 
-	s = getservbyname(ptr, "tcp");
-	if (s != NULL)
-		port = ntohs(s->s_port);
-	else
-		port = atoi(ptr);
-
-	return port;
+	return parse_port(port);
 }
 
 static char *progname(char *arg0)
@@ -373,7 +379,7 @@ static void parse_args(int argc, char *argv[])
 
 		case 129:	/* --cport=80 */
 			compat      = 1;
-			target_port = atoi(optarg);
+			target_port = parse_port(optarg);
 			break;
 
 		case 130:	/* --laddr=127.0.0.1 */
@@ -383,7 +389,7 @@ static void parse_args(int argc, char *argv[])
 
 		case 131:	/* --lport=8080 */
 			compat     = 1;
-			local_port = atoi(optarg);
+			local_port = parse_port(optarg);
 			break;
 #endif
 		default:
@@ -708,15 +714,16 @@ static int target_init(char *addr, int port, struct sockaddr_in *target)
 	if (addr) {
 		struct hostent *hp;
 
-		syslog(LOG_DEBUG, "target is %s", addr);
 		hp = gethostbyname(addr);
 		if (!hp) {
 			syslog(LOG_ERR, "Unknown host %s", addr);
 			return -1;
 		}
+
+		syslog(LOG_DEBUG, "target is %s:%d", addr, port);
 		memcpy(&target->sin_addr, hp->h_addr, hp->h_length);
 	} else {
-		syslog(LOG_DEBUG, "target is default");
+		syslog(LOG_DEBUG, "target is default, 0.0.0.0:%d", port);
 		target->sin_addr.s_addr = htonl(inet_addr("0.0.0.0"));
 	}
 
@@ -761,9 +768,10 @@ static int target_connect(int client, struct sockaddr_in *target)
 			syslog(LOG_ERR, "Failed resolving outbound IP address, %s: %s", bind_addr, strerror(errno));
 			return -1;
 		}
-		memcpy(&addr_out.sin_addr, hp->h_addr, hp->h_length);
 
+		memcpy(&addr_out.sin_addr, hp->h_addr, hp->h_length);
 		syslog(LOG_DEBUG, "IP address for target is %s", inet_ntoa(addr_out.sin_addr));
+
 	}
 
 	sd = socket(AF_INET, SOCK_STREAM, 0);
@@ -907,8 +915,8 @@ static int server_socket(char *addr, int port, int fail)
 	if (addr != NULL) {
 		struct hostent *hp;
 	  
-		syslog(LOG_DEBUG, "listening on %s", addr);
-		if ((hp = gethostbyname(addr)) == NULL) {
+		hp = gethostbyname(addr);
+		if (!hp) {
 			if (fail) {
 				close(sd);
 				return -1;
@@ -917,9 +925,11 @@ static int server_socket(char *addr, int port, int fail)
 			syslog(LOG_ERR, "Cannot resolve hostname %s: %s", addr, strerror(errno));
 			exit(1);
 		}
+
+		syslog(LOG_DEBUG, "listening on %s:%d", addr, port);
 		memcpy(&server.sin_addr, hp->h_addr, hp->h_length);
 	} else {
-		syslog(LOG_DEBUG, "local IP is default");
+		syslog(LOG_DEBUG, "local IP is default, listening on 0.0.0.0:%d", port);
 		server.sin_addr.s_addr = htonl(inet_addr("0.0.0.0"));
 	}
      
